@@ -139,6 +139,10 @@ getdns_return_t
 getdns_context_get_dns_transport(getdns_context *context,
     getdns_transport_t* value);
 
+getdns_return_t
+getdns_context_get_dns_transport_list(getdns_context *context,
+    size_t* transport_count, getdns_transport_list_t **transports);
+
 getdns_return_t
 getdns_context_get_limit_outstanding_queries(getdns_context *context,
     uint16_t* limit);
@@ -146,6 +150,9 @@ getdns_context_get_limit_outstanding_queries(getdns_context *context,
 getdns_return_t
 getdns_context_get_timeout(getdns_context *context, uint64_t* timeout);
 
+getdns_return_t
+getdns_context_get_idle_timeout(getdns_context *context, uint64_t* timeout);
+
 getdns_return_t
 getdns_context_get_follow_redirects(getdns_context *context,
     getdns_redirects_t* value);
@@ -332,6 +339,15 @@ const char *getdns_get_errorstr_by_id(uint16_t err);
  */
 getdns_return_t getdns_strerror(getdns_return_t err, char *buf, size_t buflen);
 
+#define GETDNS_VERSION "0.3.0"
+#define GETDNS_NUMERIC_VERSION 0x00030000
+#define GETDNS_API_VERSION "July 2015"
+#define GETDNS_API_NUMERIC_VERSION 0x07df0700
+
+const char *getdns_get_version(void);
+uint32_t getdns_get_version_number(void);
+const char *getdns_get_api_version(void);
+uint32_t getdns_get_api_version_number(void);
 
 #ifdef __cplusplus
 }
