@@ -255,9 +255,9 @@ static priv_getdns_rdata_def        soa_rdata[] = {
 	{ "rname"                       , GETDNS_RDF_N_C  },
 	{ "serial"                      , GETDNS_RDF_I4   },
 	{ "refresh"                     , GETDNS_RDF_I4   },
-	{ "refresh"                     , GETDNS_RDF_I4   },
 	{ "retry"                       , GETDNS_RDF_I4   },
-	{ "expire"                      , GETDNS_RDF_I4   }};
+	{ "expire"                      , GETDNS_RDF_I4   },
+	{ "minimum"                     , GETDNS_RDF_I4   }};
 static priv_getdns_rdata_def         mg_rdata[] = {
 	{ "mgmname"                     , GETDNS_RDF_N_C  }};
 static priv_getdns_rdata_def         mr_rdata[] = {
@@ -408,6 +408,10 @@ static priv_getdns_rdata_def        hip_rdata[] = {
 	{ "hit"                         , GETDNS_RDF_SPECIAL, &hip_hit },
 	{ "public_key"                  , GETDNS_RDF_SPECIAL, &hip_public_key },
 	{ "rendezvous_servers"          , GETDNS_RDF_N_M  }};
+static priv_getdns_rdata_def        csync_rdata[] = {
+	{ "serial"                      , GETDNS_RDF_I4   },
+	{ "flags"                       , GETDNS_RDF_I2   },
+	{ "type_bit_maps"               , GETDNS_RDF_X    }};
 static priv_getdns_rdata_def        spf_rdata[] = {
 	{ "text"                        , GETDNS_RDF_S_M  }};
 static priv_getdns_rdata_def        nid_rdata[] = {
@@ -519,7 +523,7 @@ static priv_getdns_rr_def priv_getdns_rr_defs[] = {
 	{        "CDS",         ds_rdata, ALEN(        ds_rdata) },
 	{    "CDNSKEY",     dnskey_rdata, ALEN(    dnskey_rdata) },
 	{ "OPENPGPKEY",    UNKNOWN_RDATA, 0                      }, /* 61 - */
-	{      "CSYNC",    UNKNOWN_RDATA, 0                      }, /* - 62 */
+	{      "CSYNC",      csync_rdata, ALEN(     csync_rdata) }, /* - 62 */
 	{         NULL,             NULL, 0                      },
 	{         NULL,             NULL, 0                      },
 	{         NULL,             NULL, 0                      },
@@ -737,627 +741,95 @@ priv_getdns_rr_type_name(int rr_type)
 	return priv_getdns_rr_def_lookup(rr_type)->name;
 }
 
-/* list of txt records */
-static getdns_return_t
-priv_getdns_equip_dict_with_txt_rdfs(struct getdns_dict* rdata, ldns_rr* rr,
-                                     const priv_getdns_rr_def* def,
-                                     struct getdns_context* context) {
-    size_t i;
-    struct getdns_bindata bindata;
-    uint8_t buffer[LDNS_MAX_RDFLEN];
-    getdns_return_t r = GETDNS_RETURN_GOOD;
-    struct getdns_list* records = getdns_list_create_with_context(context);
-    if (!records) {
-        return GETDNS_RETURN_MEMORY_ERROR;
-    }
-    for (i = 0; i < ldns_rr_rd_count(rr) && r == GETDNS_RETURN_GOOD; ++i) {
-        ldns_rdf* rdf = ldns_rr_rdf(rr, i);
-        int rdf_size = (int) ldns_rdf_size(rdf);
-        uint8_t* rdf_data = ldns_rdf_data(rdf);
-        if (rdf_size < 1) {
-            r = GETDNS_RETURN_GENERIC_ERROR;
-            continue;
-        }
-        int txt_size = (int) rdf_data[0];
-        if (rdf_size < txt_size) {
-            r = GETDNS_RETURN_GENERIC_ERROR;
-            continue;
-        }
-        bindata.size = txt_size + 1;
-        memcpy(buffer, rdf_data + 1, txt_size);
-        buffer[txt_size] = 0;
-        bindata.data = buffer;
-
-        r = getdns_list_set_bindata(records, i, &bindata);
-    }
-    if (r == GETDNS_RETURN_GOOD) {
-        r = getdns_dict_set_list(rdata, def->rdata[0].name, records);
-    }
-    getdns_list_destroy(records);
-    return r;
-}
-
-/* heavily borrowed/copied from ldns 1.6.17 */
-static
-getdns_return_t getdns_rdf_hip_get_alg_hit_pk(ldns_rdf *rdf, uint8_t* alg,
-                                              struct getdns_bindata* hit,
-                                              struct getdns_bindata* pk)
-{
-    uint8_t *data;
-    size_t rdf_size;
-
-    if ((rdf_size = ldns_rdf_size(rdf)) < 6) {
-        return GETDNS_RETURN_GENERIC_ERROR;
-    }
-    data = ldns_rdf_data(rdf);
-    hit->size = data[0];
-    *alg      = data[1];
-    pk->size  = ldns_read_uint16(data + 2);
-    hit->data      = data + 4;
-    pk->data       = data + 4 + hit->size;
-    if (hit->size == 0 || pk->size == 0 ||
-        rdf_size < (size_t) hit->size + pk->size + 4) {
-        return GETDNS_RETURN_GENERIC_ERROR;
-    }
-    return GETDNS_RETURN_GOOD;
-}
-
-static getdns_return_t
-priv_getdns_equip_dict_with_hip_rdfs(struct getdns_dict* rdata, ldns_rr* rr,
-                                     const priv_getdns_rr_def* def,
-                                     struct getdns_context* context) {
-    uint8_t alg;
-    getdns_return_t r;
-    struct getdns_bindata hit_data;
-    struct getdns_bindata key_data;
-    /* first rdf contains the key data */
-    ldns_rdf* rdf = ldns_rr_rdf(rr, 0);
-    /* ask LDNS to parse it for us */
-    r = getdns_rdf_hip_get_alg_hit_pk(rdf, &alg, &hit_data, &key_data);
-    if (r != GETDNS_RETURN_GOOD) {
-        return GETDNS_RETURN_GENERIC_ERROR;
-    }
-
-    r = getdns_dict_set_int(rdata, def->rdata[0].name, alg);
-    r |= getdns_dict_set_bindata(rdata, def->rdata[1].name, &hit_data);
-    r |= getdns_dict_set_bindata(rdata, def->rdata[2].name, &key_data);
-    if (r != GETDNS_RETURN_GOOD) {
-        return GETDNS_RETURN_GENERIC_ERROR;
-    }
-
-    if (ldns_rr_rd_count(rr) > 1) {
-        /* servers */
-        size_t i;
-        struct getdns_bindata server_data;
-        struct getdns_list* servers = getdns_list_create_with_context(context);
-        if (!servers) {
-            return GETDNS_RETURN_MEMORY_ERROR;
-        }
-        for (i = 1; i < ldns_rr_rd_count(rr) && r == GETDNS_RETURN_GOOD; ++i) {
-            ldns_rdf* server_rdf = ldns_rr_rdf(rr, i);
-            server_data.size = ldns_rdf_size(server_rdf);
-            server_data.data = ldns_rdf_data(server_rdf);
-            r = getdns_list_set_bindata(servers, i - 1, &server_data);
-        }
-        if (r == GETDNS_RETURN_GOOD) {
-            r = getdns_dict_set_list(rdata, def->rdata[3].name, servers);
-        }
-        /* always clean up */
-        getdns_list_destroy(servers);
-        if (r != GETDNS_RETURN_GOOD) {
-        	return GETDNS_RETURN_GENERIC_ERROR;
-        }
-    }
-
-    return r;
-}
-
-static getdns_return_t
-priv_append_apl_record(getdns_list* records, ldns_rdf* rdf,
-    const priv_getdns_rr_def* def, getdns_context* context)
-{
-    getdns_return_t r = GETDNS_RETURN_GOOD;
-    uint8_t* data;
-    size_t size;
-    uint16_t family;
-    uint8_t prefix;
-    uint8_t negation;
-    size_t addr_len;
-    size_t pos = 0;
-    size_t index = 0;
-    struct getdns_bindata addr_data;
-
-    if (ldns_rdf_get_type(rdf) != LDNS_RDF_TYPE_APL) {
-        return GETDNS_RETURN_GENERIC_ERROR;
-    }
-    getdns_list_get_length(records, &index);
-
-    data = ldns_rdf_data(rdf);
-    size = ldns_rdf_size(rdf);
-    if (size < 4) {
-        /* not enough for the fam, prefix, n, and data len */
-        return GETDNS_RETURN_GENERIC_ERROR;
-    }
-    while (pos < size && r == GETDNS_RETURN_GOOD) {
-        struct getdns_dict* apl_dict;
-        family = ldns_read_uint16(data + pos);
-        prefix = data[pos + 2];
-        negation = (data[pos + 3] & 0x80) > 1 ? 1 : 0;
-        addr_len = data[pos + 3] & 0x7F;
-        if (size < 4 + addr_len) {
-            /* not enough.. */
-            return GETDNS_RETURN_GENERIC_ERROR;
-        }
-        addr_data.size = addr_len;
-        addr_data.data = data + 4 + pos;
-
-        /* add to a dictionary */
-        apl_dict = getdns_dict_create_with_context(context);
-        if (!apl_dict) {
-            /* memory fail */
-            return GETDNS_RETURN_MEMORY_ERROR;
-        }
-        r |= getdns_dict_set_int(apl_dict, def->rdata[1].name, family);
-        r |= getdns_dict_set_int(apl_dict, def->rdata[2].name, prefix);
-        r |= getdns_dict_set_int(apl_dict, def->rdata[3].name, negation);
-        r |= getdns_dict_set_bindata(apl_dict, def->rdata[4].name, &addr_data);
-
-        if (r == GETDNS_RETURN_GOOD) {
-            r = getdns_list_set_dict(records, index, apl_dict);
-        }
-        pos += addr_data.size + 4;
-        ++index;
-        /* always clean up */
-        getdns_dict_destroy(apl_dict);
-    }
-
-    return r;
-}
-
-static getdns_return_t
-priv_getdns_equip_dict_with_apl_rdfs(struct getdns_dict* rdata, ldns_rr* rr,
-                                     const priv_getdns_rr_def* def,
-                                     struct getdns_context* context) {
-    size_t i;
-    getdns_return_t r = GETDNS_RETURN_GOOD;
-    struct getdns_list* records = getdns_list_create_with_context(context);
-    if (!records) {
-        return GETDNS_RETURN_MEMORY_ERROR;
-    }
-    for (i = 0; i < ldns_rr_rd_count(rr) && r == GETDNS_RETURN_GOOD; ++i) {
-        r = priv_append_apl_record(records, ldns_rr_rdf(rr, i),
-                                   def, context);
-    }
-    if (r == GETDNS_RETURN_GOOD) {
-        getdns_dict_set_list(rdata, def->rdata[0].name, records);
-    }
-    getdns_list_destroy(records);
-
-    return GETDNS_RETURN_GOOD;
-}
-
-static getdns_return_t
-priv_getdns_equip_dict_with_spf_rdfs(struct getdns_dict* rdata, ldns_rr* rr,
-                                     const priv_getdns_rr_def* def,
-                                     struct getdns_context* context) {
-    size_t i;
-    struct getdns_bindata bindata;
-    getdns_return_t r = GETDNS_RETURN_GOOD;
-    int num_copied = 0;
-    bindata.size = 0;
-    /* one giant bindata */
-    /* validate and calculate size */
-    for (i = 0; i < ldns_rr_rd_count(rr) && r == GETDNS_RETURN_GOOD; ++i) {
-        ldns_rdf* rdf = ldns_rr_rdf(rr, i);
-        int rdf_size = (int) ldns_rdf_size(rdf);
-        uint8_t* rdf_data = ldns_rdf_data(rdf);
-        if (rdf_size < 1) {
-            r = GETDNS_RETURN_GENERIC_ERROR;
-            continue;
-        }
-        /* txt size without null byte */
-        int txt_size = (int) rdf_data[0];
-        if (rdf_size < txt_size) {
-            r = GETDNS_RETURN_GENERIC_ERROR;
-            continue;
-        }
-        bindata.size += txt_size;
-    }
-    /* add one for the null byte */
-    bindata.size++;
-
-    if (r != GETDNS_RETURN_GOOD) {
-        /* validations failed */
-        return r;
-    }
-	bindata.data = context
-	    ? GETDNS_XMALLOC(context->my_mf, uint8_t, bindata.size)
-	    : malloc(bindata.size);
-    if (!bindata.data) {
-        return GETDNS_RETURN_MEMORY_ERROR;
-    }
-    /* copy in */
-    for (i = 0; i < ldns_rr_rd_count(rr) && r == GETDNS_RETURN_GOOD; ++i) {
-        ldns_rdf* rdf = ldns_rr_rdf(rr, i);
-        /* safe to trust these now */
-        uint8_t* rdf_data = ldns_rdf_data(rdf);
-        int txt_size = (int) rdf_data[0];
-        memcpy(bindata.data + num_copied, rdf_data + 1, txt_size);
-        num_copied += txt_size;
-    }
-    bindata.data[num_copied] = 0;
-    r = getdns_dict_set_bindata(rdata, def->rdata[0].name, &bindata);
-	if (context)
-		GETDNS_FREE(context->my_mf, bindata.data);
-	else
-		free(bindata.data);
-    return r;
-}
-
-
-static getdns_return_t
-priv_getdns_equip_dict_with_rdfs(struct getdns_dict *rdata, ldns_rr *rr,
-                                 struct getdns_context* context)
-{
-	getdns_return_t r = GETDNS_RETURN_GOOD;
-	const priv_getdns_rr_def *def;
-	struct getdns_bindata bindata;
-	size_t i;
-
-	struct getdns_bindata *rdata_raw;
-	const char *sptr;
-	char *dptr, tmpbuf[100];
-
-	uint8_t *rdf_data;
-	size_t   rdf_size;
-
-	assert(rdata);
-	assert(rr);
-
-	def = priv_getdns_rr_def_lookup(ldns_rr_get_type(rr));
-    /* specialty handlers */
-    /* TODO: convert generic one into function w/ similar signature and store in the
-     * def? */
-    if (def->rdata == txt_rdata) {
-        return priv_getdns_equip_dict_with_txt_rdfs(rdata, rr, def, context);
-    } else if (def->rdata == hip_rdata) {
-        return priv_getdns_equip_dict_with_hip_rdfs(rdata, rr, def, context);
-    } else if (def->rdata == apl_rdata) {
-        return priv_getdns_equip_dict_with_apl_rdfs(rdata, rr, def, context);
-    } else if (def->rdata == spf_rdata) {
-        return priv_getdns_equip_dict_with_spf_rdfs(rdata, rr, def, context);
-    } else if (def->name &&
-	    strlen(def->name) <= sizeof(tmpbuf) - 9 /* strlen("_unknown")+1 */ &&
-	    (def->rdata == UNKNOWN_RDATA ||
-	    ldns_rr_descriptor_field_type(
-		ldns_rr_descript(ldns_rr_get_type(rr)), 0) == LDNS_RDF_TYPE_UNKNOWN)) {
-
-		r = getdns_dict_get_bindata(rdata, "rdata_raw", &rdata_raw);
-		if (r != GETDNS_RETURN_GOOD)
-			return r;
-
-		sptr = def->name;
-		dptr = tmpbuf;
-		do *dptr++ = tolower(*sptr);
-			while (*sptr++); /* Including terminating '\0' */
-
-		return getdns_dict_set_bindata(
-		    rdata, strcat(tmpbuf, "_unknown"), rdata_raw);
-    }
-    /* generic */
-	if (ldns_rr_rd_count(rr) != def->n_rdata_fields)
-		return r;
-
-	for (i = 0; !r && i < ldns_rr_rd_count(rr) 
-	               && i < def->n_rdata_fields; i++) {
-
-		if (! (def->rdata[i].type & GETDNS_RDF_INTEGER)) {
-			bindata.size = ldns_rdf_size(ldns_rr_rdf(rr, i));
-			bindata.data = ldns_rdf_data(ldns_rr_rdf(rr, i));
-			r = getdns_dict_set_bindata(
-			    rdata, (char*)def->rdata[i].name, &bindata);
-			continue;
-		}
-		rdf_size = ldns_rdf_size(ldns_rr_rdf(rr, i));
-		rdf_data = ldns_rdf_data(ldns_rr_rdf(rr, i));
-		r = getdns_dict_set_int(rdata, (char *)def->rdata[i].name,
-		      rdf_size == 1 ?                 *rdf_data
-		    : rdf_size == 2 ? ldns_read_uint16(rdf_data)
-		    : rdf_size == 4 ? ldns_read_uint32(rdf_data) : -1);
-	}
-	return r;
-}
-
-static getdns_return_t
-priv_getdns_create_dict_from_rdfs(
-    struct getdns_context *context, ldns_rr *rr, struct getdns_dict** rdata)
-{
-	getdns_return_t r = GETDNS_RETURN_GOOD;
-	struct getdns_bindata rdata_raw;
-	uint8_t *data_ptr;
-	size_t i;
-
-	assert(rr);
-	assert(rdata);
-
-	*rdata = getdns_dict_create_with_context(context);
-	if (! *rdata)
-		return GETDNS_RETURN_MEMORY_ERROR;
-	do { /* break on error (to cleanup *rdata) */
-
-		/* Count and reserve "raw" rdata space */
-		rdata_raw.size = 0;
-		for (i = 0; i < ldns_rr_rd_count(rr); i++)
-			rdata_raw.size += ldns_rdf_size(ldns_rr_rdf(rr, i));
-		rdata_raw.data = context
-		    ? GETDNS_XMALLOC(context->mf, uint8_t, rdata_raw.size)
-		    : malloc(rdata_raw.size);
-		if (! rdata_raw.data) {
-			r = GETDNS_RETURN_MEMORY_ERROR;
-			break;
-		}
-		/* Copy rdata fields to rdata space */
-		data_ptr = rdata_raw.data;
-		for (i = 0; i < ldns_rr_rd_count(rr); i++) {
-			(void) memcpy(data_ptr,
-			    ldns_rdf_data(ldns_rr_rdf(rr, i)),
-			    ldns_rdf_size(ldns_rr_rdf(rr, i)));
-			data_ptr += ldns_rdf_size(ldns_rr_rdf(rr, i));
-		}
-
-		/* Set "rdata_raw" attribute" */
-		r = getdns_dict_set_bindata(*rdata, "rdata_raw", &rdata_raw);
-		if (context)
-			GETDNS_FREE(context->mf, rdata_raw.data);
-		else
-			free(rdata_raw.data);
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-
-		/* Now set the RR type specific attributes */
-		r = priv_getdns_equip_dict_with_rdfs(*rdata, rr, context);
-		if (r == GETDNS_RETURN_GOOD)
-			return r;
-	} while(0);
-	getdns_dict_destroy(*rdata);
-	return r;
-}
-
 getdns_return_t
-priv_getdns_create_dict_from_rr(
-    struct getdns_context *context, ldns_rr *rr, struct getdns_dict** rr_dict)
-{
-	getdns_return_t r = GETDNS_RETURN_GOOD;
-	struct getdns_bindata name;
-	struct getdns_dict *rdata;
-
-	assert(rr);
-	assert(rr_dict);
-
-	*rr_dict = getdns_dict_create_with_context(context);
-	if (! *rr_dict)
-		return GETDNS_RETURN_MEMORY_ERROR;
-	do { /* break on error (to cleanup *rr_dict) */
-		r = getdns_dict_set_int(*rr_dict,
-		    "type", ldns_rr_get_type(rr));
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-		r = getdns_dict_set_int(*rr_dict,
-		    "class", ldns_rr_get_class(rr));
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-		r = getdns_dict_set_int(*rr_dict, "ttl", ldns_rr_ttl(rr));
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-
-		/* "name" attribute.
-		 * ldns_rr_owner(rr) is already uncompressed!
-		 */
-		name.size = ldns_rdf_size(ldns_rr_owner(rr));
-		name.data = ldns_rdf_data(ldns_rr_owner(rr));
-		r = getdns_dict_set_bindata(*rr_dict, "name", &name);
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-
-		/* The "rdata" dict... copies of copies of copies :( */
-		r = priv_getdns_create_dict_from_rdfs(context, rr, &rdata);
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-		r = getdns_dict_set_dict(*rr_dict, "rdata", rdata);
-		getdns_dict_destroy(rdata);
-		if (r == GETDNS_RETURN_GOOD)
-			return r;
-	} while (0);
-	getdns_dict_destroy(*rr_dict);
-	return r;
-}
-
-static getdns_return_t priv_getdns_construct_wire_rdata_from_rdata(
-    struct getdns_dict *rdata, uint32_t rr_type,
-    uint8_t **wire, size_t *wire_size)
-{
-	getdns_return_t r = GETDNS_RETURN_GOOD;
-	const ldns_rr_descriptor *rr_descript;
-	const priv_getdns_rr_def *def;
-	size_t i, size;
-	struct getdns_bindata *bindata;
-	uint32_t value;
-	uint8_t *ptr;
-
-	assert(rdata);
-	assert(wire);
-	assert(wire_size);
-
-	def = priv_getdns_rr_def_lookup(rr_type);
-	rr_descript = ldns_rr_descript(rr_type);
-
-	/* First calculate needed size */
-	size = 0;
-	for (i = 0; !r && i < def->n_rdata_fields; i++) {
-		if (def->rdata[i].type & GETDNS_RDF_BINDATA)
-			if ((r = getdns_dict_get_bindata(rdata,
-			    def->rdata[i].name, &bindata)))
-				break;
-			else {
-				size += bindata->size;
-				continue;
-			}
-		else if (!(def->rdata[i].type & GETDNS_RDF_INTEGER)) {
-			r = GETDNS_RETURN_GENERIC_ERROR;
-			break;
-		}
-		switch (ldns_rr_descriptor_field_type(rr_descript, i)) {
-
-		case LDNS_RDF_TYPE_CLASS:
-		case LDNS_RDF_TYPE_ALG  :
-		case LDNS_RDF_TYPE_INT8 : size += 1;
-					  break;
-		case LDNS_RDF_TYPE_TYPE :
-		case LDNS_RDF_TYPE_CERT_ALG:
-		case LDNS_RDF_TYPE_INT16: size += 2;
-					  break;
-		case LDNS_RDF_TYPE_TIME :
-		case LDNS_RDF_TYPE_PERIOD:
-		case LDNS_RDF_TYPE_INT32: size += 4;
-					  break;
-		default: r = GETDNS_RETURN_GENERIC_ERROR;
-		         break;
-		}
-	}
-	*wire_size = size + 2;
-	*wire = ptr = GETDNS_XMALLOC(rdata->mf, uint8_t, size + 2);
-	if (! ptr)
-		return GETDNS_RETURN_MEMORY_ERROR;
-
-	ptr[0] = (uint8_t) (size >> 8) & 0xff;
-	ptr[1] = (uint8_t)  size       & 0xff;
-	ptr += 2;
-	for (i = 0; !r && i < def->n_rdata_fields; i++) {
-		if (def->rdata[i].type & GETDNS_RDF_BINDATA)
-			if ((r = getdns_dict_get_bindata(rdata,
-			    def->rdata[i].name, &bindata)))
-				break;
-			else {
-				(void) memcpy(ptr, bindata->data,
-				                   bindata->size);
-				ptr += bindata->size;
-				continue;
-			}
-		else if (!(def->rdata[i].type & GETDNS_RDF_INTEGER)) {
-			r = GETDNS_RETURN_GENERIC_ERROR;
-			break;
-		}
-		if ((r = getdns_dict_get_int(
-		    rdata, def->rdata[i].name, &value)))
-			break;
-
-		switch (ldns_rr_descriptor_field_type(rr_descript, i)) {
-
-		case LDNS_RDF_TYPE_CLASS:
-		case LDNS_RDF_TYPE_ALG  :
-		case LDNS_RDF_TYPE_INT8 : ptr[0] = (uint8_t) value      & 0xff;
-		                          ptr += 1;
-					  break;
-		case LDNS_RDF_TYPE_TYPE :
-		case LDNS_RDF_TYPE_CERT_ALG:
-		case LDNS_RDF_TYPE_INT16: ptr[0] = (uint8_t)(value>> 8) & 0xff;
-					  ptr[1] = (uint8_t) value      & 0xff;
-					  ptr += 2;
-					  break;
-		case LDNS_RDF_TYPE_TIME :
-		case LDNS_RDF_TYPE_PERIOD:
-		case LDNS_RDF_TYPE_INT32: ptr[0] = (uint8_t)(value>>24) & 0xff;
-					  ptr[1] = (uint8_t)(value>>16) & 0xff;
-					  ptr[2] = (uint8_t)(value>>8 ) & 0xff;
-					  ptr[3] = (uint8_t) value      & 0xff;
-					  ptr += 4;
-					  break;
-		default: r = GETDNS_RETURN_GENERIC_ERROR;
-		         break;
-		}
-	}
-	if (r)
-		GETDNS_FREE(rdata->mf, ptr);
-	return r;
-}
-
-static getdns_return_t
-priv_getdns_dict_get_raw_rdata(struct getdns_dict *rdata,
-    uint8_t **wire, size_t *wire_size)
-{
-	getdns_return_t r;
-	struct getdns_bindata *bindata;
-
-	if ((r = getdns_dict_get_bindata(rdata, "rdata_raw", &bindata)))
-		return r;
-
-	*wire_size = bindata->size + 2;
-	*wire = GETDNS_XMALLOC(rdata->mf, uint8_t, *wire_size);
-	if (! *wire)
-		return GETDNS_RETURN_MEMORY_ERROR;
-
-	(*wire)[0] = (uint8_t) (bindata->size >> 8) & 0xff;
-	(*wire)[1] = (uint8_t)  bindata->size       & 0xff;
-
-	(void) memcpy(*wire + 2, bindata->data, bindata->size);
-	return GETDNS_RETURN_GOOD;
-}
-
-getdns_return_t
-priv_getdns_create_rr_from_dict(struct getdns_dict *rr_dict, ldns_rr **rr)
+priv_getdns_rr_dict2wire(getdns_dict *rr_dict, gldns_buffer *buf)
 {
 	getdns_return_t r = GETDNS_RETURN_GOOD;
 	struct getdns_bindata *name;
+	struct getdns_bindata *rdata_raw;
+	struct getdns_bindata *bindata;
 	struct getdns_dict *rdata;
 	uint32_t rr_type;
-	ldns_rdf *owner;
-	ldns_status s;
-	size_t pos;
-	uint8_t *wire;
-	size_t wire_size;
+	uint32_t rr_class = GETDNS_RRCLASS_IN;
+	uint32_t rr_ttl = 0;
+	uint32_t value;
+	const priv_getdns_rr_def *rr_def;
+	const priv_getdns_rdata_def *rd_def;
+	int n_rdata_fields;
+	size_t j, rdata_size_mark;
 
 	assert(rr_dict);
-	assert(rr);
+	assert(buf);
 
-	*rr = ldns_rr_new();
-	if (! *rr)
-		return GETDNS_RETURN_MEMORY_ERROR;
-	do {
-		r = getdns_dict_get_bindata(rr_dict, "name", &name);
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-		owner = ldns_rdf_new_frm_data(
-		    LDNS_RDF_TYPE_DNAME, name->size, name->data);
-		if (! owner) {
-			r = GETDNS_RETURN_MEMORY_ERROR;
+	if ((r = getdns_dict_get_bindata(rr_dict, "name", &name)))
+		goto error;
+	gldns_buffer_write(buf, name->data, name->size);
+
+	if ((r = getdns_dict_get_int(rr_dict, "type", &rr_type)))
+		goto error;
+	gldns_buffer_write_u16(buf, (uint16_t)rr_type);
+
+	(void) getdns_dict_get_int(rr_dict, "class", &rr_class);
+	gldns_buffer_write_u16(buf, (uint16_t)rr_class);
+
+	(void) getdns_dict_get_int(rr_dict, "ttl", &rr_ttl);
+	gldns_buffer_write_u32(buf, rr_ttl);
+
+	/* Does rdata contain compressed names?
+	 * Because rdata_raw is unusable then.
+	 */
+	rr_def = priv_getdns_rr_def_lookup(rr_type);
+	for ( rd_def = rr_def->rdata
+	    , n_rdata_fields = rr_def->n_rdata_fields
+	    ; n_rdata_fields ; n_rdata_fields-- , rd_def++ ) {
+
+		if (rd_def->type & GETDNS_RDF_COMPRESSED)
 			break;
+	}
+
+	if ((r = getdns_dict_get_dict(rr_dict, "rdata", &rdata)))
+		goto error;
+
+	if (n_rdata_fields == 0 && GETDNS_RETURN_GOOD ==
+	    (r = getdns_dict_get_bindata(rdata, "rdata_raw", &rdata_raw))) {
+
+		gldns_buffer_write_u16(buf, (uint16_t)rdata_raw->size);
+		gldns_buffer_write(buf, rdata_raw->data, rdata_raw->size);
+
+	} else if (n_rdata_fields || r == GETDNS_RETURN_NO_SUCH_DICT_NAME) {
+
+		rdata_size_mark = gldns_buffer_position(buf);
+		gldns_buffer_skip(buf, 2);
+
+		for ( rd_def = rr_def->rdata
+		    , n_rdata_fields = rr_def->n_rdata_fields
+		    ; n_rdata_fields ; n_rdata_fields-- , rd_def++ ) {
+
+			if (rd_def->type & GETDNS_RDF_BINDATA) {
+				if ((r = getdns_dict_get_bindata(rdata,
+				    rd_def->name, &bindata)))
+					break;
+
+				gldns_buffer_write(buf, bindata->data
+				                      , bindata->size );
+				continue;
+			}
+			if (!(rd_def->type & GETDNS_RDF_INTEGER)) {
+				r = GETDNS_RETURN_GENERIC_ERROR;
+				break;
+			}
+			if ((r = getdns_dict_get_int(
+			    rdata, rd_def->name, &value)))
+				break;
+			
+			for (j = rd_def->type & GETDNS_RDF_FIXEDSZ; j; j--)
+				gldns_buffer_write_u8(buf,
+				    (uint8_t)(value >> (8 * (j - 1))) & 0xff);
 		}
-		ldns_rr_set_owner(*rr, owner);
-
-		r = getdns_dict_get_int(rr_dict, "type", &rr_type);
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-		ldns_rr_set_type(*rr, rr_type);
-
-		r = getdns_dict_get_dict(rr_dict, "rdata", &rdata);
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-
-		r = priv_getdns_dict_get_raw_rdata(rdata, &wire, &wire_size);
-		if (r == GETDNS_RETURN_NO_SUCH_DICT_NAME) {
-			r = priv_getdns_construct_wire_rdata_from_rdata(
-			    rdata, rr_type, &wire, &wire_size);
-		}
-		if (r != GETDNS_RETURN_GOOD)
-			break;
-		pos = 0;
-		s = ldns_wire2rdf(*rr, wire, wire_size, &pos);
-		GETDNS_FREE(rr_dict->mf, wire);
-		if (s == LDNS_STATUS_OK)
-			return r;
-		r = GETDNS_RETURN_GENERIC_ERROR;
-	} while (0);
-	ldns_rr_free(*rr);
+		gldns_buffer_write_u16_at(buf, rdata_size_mark,
+		    (uint16_t)(gldns_buffer_position(buf)-rdata_size_mark-2));
+	}
+error:
 	return r;
 }
 
