@@ -32,6 +32,7 @@
 #include <dirent.h>
 #include <getdns/getdns.h>
 #include <getdns/getdns_extra.h>
+#include <types-internal.h>
 
 static int quiet = 0;
 static int batch_mode = 0;
@@ -92,13 +93,40 @@ ipaddr_dict(getdns_context *context, char *ipstr)
 	if (*portstr)
 		getdns_dict_set_int(r, "port", (int32_t)atoi(portstr));
 	if (*tls_portstr)
-		getdns_dict_set_int(r, "tls-port", (int32_t)atoi(tls_portstr));
+		getdns_dict_set_int(r, "tls_port", (int32_t)atoi(tls_portstr));
 	if (*scope_id_str)
 		getdns_dict_util_set_string(r, "scope_id", scope_id_str);
 
 	return r;
 }
 
+static getdns_return_t
+fill_transport_list(getdns_context *context, char *transport_list_str, 
+                    getdns_transport_list_t *transports, size_t *transport_count)
+{
+	for (size_t i = 0; i < strlen(transport_list_str); i++, (*transport_count)++) {
+		switch(*(transport_list_str + i)) {
+			case 'U': 
+				transports[i] = GETDNS_TRANSPORT_UDP;
+				break;
+			case 'T': 
+				transports[i] = GETDNS_TRANSPORT_TCP;
+				break;
+			case 'L': 
+				transports[i] = GETDNS_TRANSPORT_TLS;
+				break;
+			case 'S': 
+				transports[i] = GETDNS_TRANSPORT_STARTTLS;
+				break;
+			default:
+				fprintf(stderr, "Unrecognised transport '%c' in string %s\n", 
+				       *(transport_list_str + i), transport_list_str);
+				return GETDNS_RETURN_GENERIC_ERROR;
+		}
+	}
+	return GETDNS_RETURN_GOOD;
+}
+
 void
 print_usage(FILE *out, const char *progname)
 {
@@ -121,9 +149,11 @@ print_usage(FILE *out, const char *progname)
 	fprintf(out, "\t-J\tPretty print json response dict\n");
 	fprintf(out, "\t-p\tPretty print response dict\n");
 	fprintf(out, "\t-r\tSet recursing resolution type\n");
+	fprintf(out, "\t-R\tPrint root trust anchors\n");
 	fprintf(out, "\t-s\tSet stub resolution type (default = recursing)\n");
 	fprintf(out, "\t-S\tservice lookup (<type> is ignored)\n");
 	fprintf(out, "\t-t <timeout>\tSet timeout in miliseconds\n");
+	fprintf(out, "\t-e <idle_timeout>\tSet idle timeout in miliseconds\n");
 	fprintf(out, "\t-T\tSet transport to TCP only\n");
 	fprintf(out, "\t-O\tSet transport to TCP only keep connections open\n");
 	fprintf(out, "\t-L\tSet transport to TLS only keep connections open\n");
@@ -131,10 +161,98 @@ print_usage(FILE *out, const char *progname)
 	fprintf(out, "\t-R\tSet transport to STARTTLS with TCP fallback only keep connections open\n");
 	fprintf(out, "\t-u\tSet transport to UDP with TCP fallback\n");
 	fprintf(out, "\t-U\tSet transport to UDP only\n");
+	fprintf(out, "\t-l <transports>\tSet transport list. List can contain 1 of each of the characters\n");
+	fprintf(out, "\t\t\t U T L S for UDP, TCP, TLS or STARTTLS e.g 'UT' or 'LST' \n");
 	fprintf(out, "\t-B\tBatch mode. Schedule all messages before processing responses.\n");
 	fprintf(out, "\t-q\tQuiet mode - don't print response\n");
 }
 
+static getdns_return_t validate_chain(getdns_dict *response)
+{
+	getdns_return_t r;
+	getdns_list *validation_chain;
+	getdns_list *replies_tree;
+	getdns_dict *reply;
+	getdns_list *to_validate;
+	getdns_list *trust_anchor;
+	size_t i;
+	int s;
+	
+	if (!(to_validate = getdns_list_create()))
+		return GETDNS_RETURN_MEMORY_ERROR;
+
+	trust_anchor = getdns_root_trust_anchor(NULL);
+
+	if ((r = getdns_dict_get_list(
+	    response, "validation_chain", &validation_chain)))
+		goto error;
+
+	if ((r = getdns_dict_get_list(
+	    response, "replies_tree", &replies_tree)))
+		goto error;
+
+	fprintf(stdout, "replies_tree dnssec_status: ");
+	switch ((s = getdns_validate_dnssec(
+	    replies_tree, validation_chain, trust_anchor))) {
+
+	case GETDNS_DNSSEC_SECURE:
+		fprintf(stdout, "GETDNS_DNSSEC_SECURE\n");
+		break;
+	case GETDNS_DNSSEC_BOGUS:
+		fprintf(stdout, "GETDNS_DNSSEC_BOGUS\n");
+		break;
+	case GETDNS_DNSSEC_INDETERMINATE:
+		fprintf(stdout, "GETDNS_DNSSEC_INDETERMINATE\n");
+		break;
+	case GETDNS_DNSSEC_INSECURE:
+		fprintf(stdout, "GETDNS_DNSSEC_INSECURE\n");
+		break;
+	case GETDNS_DNSSEC_NOT_PERFORMED:
+		fprintf(stdout, "GETDNS_DNSSEC_NOT_PERFORMED\n");
+		break;
+	default:
+		fprintf(stdout, "%d\n", (int)s);
+	}
+
+	i = 0;
+	while (!(r = getdns_list_get_dict(replies_tree, i++, &reply))) {
+
+		if ((r = getdns_list_set_dict(to_validate, 0, reply)))
+			goto error;
+
+		fprintf( stdout
+		       , "reply %zu, dnssec_status: ", i);
+		switch ((s = getdns_validate_dnssec(
+		    to_validate, validation_chain, trust_anchor))) {
+
+		case GETDNS_DNSSEC_SECURE:
+			fprintf(stdout, "GETDNS_DNSSEC_SECURE\n");
+			break;
+		case GETDNS_DNSSEC_BOGUS:
+			fprintf(stdout, "GETDNS_DNSSEC_BOGUS\n");
+			break;
+		case GETDNS_DNSSEC_INDETERMINATE:
+			fprintf(stdout, "GETDNS_DNSSEC_INDETERMINATE\n");
+			break;
+		case GETDNS_DNSSEC_INSECURE:
+			fprintf(stdout, "GETDNS_DNSSEC_INSECURE\n");
+			break;
+		case GETDNS_DNSSEC_NOT_PERFORMED:
+			fprintf(stdout, "GETDNS_DNSSEC_NOT_PERFORMED\n");
+			break;
+		default:
+			fprintf(stdout, "%d\n", (int)s);
+		}
+	}
+	if (r == GETDNS_RETURN_NO_SUCH_LIST_ITEM)
+		r = GETDNS_RETURN_GOOD;
+error:
+	getdns_list_destroy(trust_anchor);
+	getdns_list_destroy(to_validate);
+
+	return GETDNS_RETURN_GOOD;
+}
+
 void callback(getdns_context *context, getdns_callback_type_t callback_type,
     getdns_dict *response, void *userarg, getdns_transaction_t trans_id)
 {
@@ -147,26 +265,31 @@ void callback(getdns_context *context, getdns_callback_type_t callback_type,
 		  : getdns_pretty_print_dict(response))) {
 
 			fprintf(stdout, "ASYNC response:\n%s\n", response_str);
+			validate_chain(response);
 			free(response_str);
 		}
-		fprintf(stderr,
-			"The callback with ID %llu  was successfull.\n",
+		fprintf(stdout,
+			"Result:      The callback with ID %llu  was successfull.\n",
 			(unsigned long long)trans_id);
 
 	} else if (callback_type == GETDNS_CALLBACK_CANCEL)
 		fprintf(stderr,
-			"The callback with ID %llu was cancelled. Exiting.\n",
+			"Result:      The callback with ID %llu was cancelled. Exiting.\n",
 			(unsigned long long)trans_id);
-	else
+	else {
 		fprintf(stderr,
-			"The callback got a callback_type of %d. Exiting.\n",
+			"Result:      The callback got a callback_type of %d. Exiting.\n",
 			callback_type);
-
+		fprintf(stderr,
+			"Error :      '%s'\n",
+			getdns_get_errorstr_by_id(callback_type));
+	}
 	getdns_dict_destroy(response);
 	response = NULL;
 }
 
 #define CONTINUE ((getdns_return_t)-2)
+#define CONTINUE_ERROR ((getdns_return_t)-3)
 
 static getdns_return_t set_cookie(getdns_dict *exts, char *cookie)
 {
@@ -204,7 +327,7 @@ static getdns_return_t set_cookie(getdns_dict *exts, char *cookie)
 	}
 	bindata.data = data;
 	bindata.size = i;
-	if ((r = getdns_dict_set_int(option, "option_code", 65001)))
+	if ((r = getdns_dict_set_int(option, "option_code", 10)))
 		goto done;
 	if ((r = getdns_dict_set_bindata(option, "option_data", &bindata)))
 		goto done;
@@ -225,8 +348,9 @@ getdns_return_t parse_args(int argc, char **argv)
 	getdns_return_t r = GETDNS_RETURN_GOOD;
 	size_t i;
 	char *arg, *c, *endptr;
-	int t, print_api_info = 0;
+	int t, print_api_info = 0, print_trust_anchors = 0;
 	getdns_list *upstream_list = NULL;
+	getdns_list *tas = NULL;
 	size_t upstream_count = 0;
 
 	for (i = 1; i < argc; i++) {
@@ -243,6 +367,11 @@ getdns_return_t parse_args(int argc, char **argv)
 					    " %d", r);
 					break;
 				}
+			} else if (arg[1] == '0') {
+			    /* Unset all existing extensions*/
+				getdns_dict_destroy(extensions);
+				extensions = getdns_dict_create();
+				break;
 			} else if ((r = getdns_dict_set_int(extensions, arg+1,
 			    GETDNS_EXTENSION_TRUE))) {
 				fprintf(stderr, "Could not set extension "
@@ -328,6 +457,9 @@ getdns_return_t parse_args(int argc, char **argv)
 			case 'J':
 				json = 1;
 				break;
+			case 'k':
+				print_trust_anchors = 1;
+				break;
 			case 'p':
 				json = 0;
 			case 'q':
@@ -347,20 +479,36 @@ getdns_return_t parse_args(int argc, char **argv)
 				break;
 			case 't':
 				if (c[1] != 0 || ++i >= argc || !*argv[i]) {
-					fprintf(stderr, "ttl expected "
+					fprintf(stderr, "timeout expected "
 					    "after -t\n");
 					return GETDNS_RETURN_GENERIC_ERROR;
 				}
 				timeout = strtol(argv[i], &endptr, 10);
 				if (*endptr || timeout < 0) {
 					fprintf(stderr, "positive "
-					    "numeric ttl expected "
+					    "numeric timeout expected "
 					    "after -t\n");
 					return GETDNS_RETURN_GENERIC_ERROR;
 				}
 				getdns_context_set_timeout(
 					context, timeout);
 				goto next;
+			case 'e':
+				if (c[1] != 0 || ++i >= argc || !*argv[i]) {
+					fprintf(stderr, "idle timeout expected "
+					    "after -t\n");
+					return GETDNS_RETURN_GENERIC_ERROR;
+				}
+				timeout = strtol(argv[i], &endptr, 10);
+				if (*endptr || timeout < 0) {
+					fprintf(stderr, "positive "
+					    "numeric idle timeout expected "
+					    "after -t\n");
+					return GETDNS_RETURN_GENERIC_ERROR;
+				}
+				getdns_context_set_idle_timeout(
+					context, timeout);
+				goto next;
 			case 'T':
 				getdns_context_set_dns_transport(context,
 				    GETDNS_TRANSPORT_TCP_ONLY);
@@ -389,6 +537,21 @@ getdns_return_t parse_args(int argc, char **argv)
 				getdns_context_set_dns_transport(context,
 				    GETDNS_TRANSPORT_UDP_ONLY);
 				break;
+			case 'l':
+				if (c[1] != 0 || ++i >= argc || !*argv[i]) {
+					fprintf(stderr, "transport list expected "
+					    "after -l\n");
+					return GETDNS_RETURN_GENERIC_ERROR;
+				}
+				size_t transport_count = 0;
+				getdns_transport_list_t transports[GETDNS_TRANSPORTS_MAX];
+				if ((r = fill_transport_list(context, argv[i], transports, &transport_count)) ||
+				    (r = getdns_context_set_dns_transport_list(context, 
+				                                               transport_count, transports))){
+						fprintf(stderr, "Could not set transports\n");
+						return r;
+				}
+				break;
 			case 'B':
 				batch_mode = 1;
 			break;
@@ -416,6 +579,13 @@ next:		;
 		    getdns_context_get_api_information(context)));
 		return CONTINUE;
 	}
+	if (print_trust_anchors) {
+		if ((tas = getdns_root_trust_anchor(NULL))) {
+			fprintf(stdout, "%s\n", getdns_pretty_print_list(tas));
+			return CONTINUE;
+		} else
+			return CONTINUE_ERROR;
+	}
 	return r;
 }
 
@@ -460,8 +630,10 @@ main(int argc, char **argv)
 				if (!fgets(line, 1024, stdin) || !*line)
 					break;
 			} else {
-				if (!fgets(line, 1024, fp) || !*line)
+				if (!fgets(line, 1024, fp) || !*line) {
+					fprintf(stdout,"End of file.");
 					break;
+				}
 				fprintf(stdout,"Found query: %s", line);
 			}
 
@@ -469,11 +641,15 @@ main(int argc, char **argv)
 			linec = 1;
 			if ( ! (token = strtok(line, " \t\f\n\r")))
 				continue;
+			if (*token == '#') {
+				fprintf(stdout,"Result:      Skipping comment\n");
+					continue;
+			}
 			do linev[linec++] = token;
 			while (linec < 256 &&
 			    (token = strtok(NULL, " \t\f\n\r")));
 			if ((r = parse_args(linec, linev))) {
-				if (r == CONTINUE)
+				if (r == CONTINUE || r == CONTINUE_ERROR)
 					continue;
 				else
 					goto done_destroy_context;
@@ -534,25 +710,29 @@ main(int argc, char **argv)
 				r = GETDNS_RETURN_GENERIC_ERROR;
 				break;
 			}
-			if (r)
-				goto done_destroy_extensions;
-			if (!quiet) {
+			if (response && !quiet) {
 				if ((response_str = json ?
 				    getdns_print_json_dict(response, json == 1)
 				  : getdns_pretty_print_dict(response))) {
 
 					fprintf( stdout, "SYNC response:\n%s\n"
 					       , response_str);
+					validate_chain(response);
 					free(response_str);
 				} else {
 					r = GETDNS_RETURN_MEMORY_ERROR;
 					fprintf( stderr
 					       , "Could not print response\n");
 				}
-			} else if (r == GETDNS_RETURN_GOOD)
-				fprintf(stdout, "Response code was: GOOD\n");
-			else if (interactive)
-				fprintf(stderr, "An error occurred: %d\n", r);
+			}
+			if (r == GETDNS_RETURN_GOOD) {
+				uint32_t status;
+				getdns_dict_get_int(response, "status", &status);
+				fprintf(stdout, "Response code was: GOOD. Status was: %s\n", 
+				         getdns_get_errorstr_by_id(status));
+			} else
+				fprintf(stderr, "An error occurred: %d '%s'\n", r,
+				         getdns_get_errorstr_by_id(r));
 		}
 	} while (interactive);
 
@@ -571,8 +751,9 @@ done_destroy_context:
 
 	if (r == CONTINUE)
 		return 0;
-	if (r)
-		fprintf(stderr, "An error occurred: %d\n", r);
+	else if (r == CONTINUE_ERROR)
+		return 1;
+	fprintf(stdout, "\nAll done.\n");
 	return r;
 }
 
@@ -642,6 +823,12 @@ int get_rrtype(const char *t) {
 	                    if ((t[2]|0x20) == 'a' && (t[3]|0x20) == 'm' && (t[4]|0x20) == 'e' && t[5] == '\0')
 	                              return GETDNS_RRTYPE_CNAME;
 	                    return -1;
+	          case 'S':
+	          case 's': /* before "CS", final "YNC" (GETDNS_RRTYPE_CSYNC) */
+	                    if ((t[2]|0x20) == 'y' && (t[3]|0x20) == 'n' && (t[4]|0x20) == 'c' && t[5] == '\0')
+	                              return GETDNS_RRTYPE_CSYNC;
+	                    return -1;
+
 	          default : return -1;
 	          };
 	case 'D':
